from typing import Optional

from pydantic import BaseModel

from wexample_app.common.abstract_kernel import AbstractKernel
from wexample_app.common.abstract_kernel_child import AbstractKernelChild
from wexample_filestate.mixins.with_workdir_mixin import WithWorkdirMixin
from wexample_helpers.classes.mixin.has_snake_short_class_name_class_mixin import HasSnakeShortClassNameClassMixin
from wexample_helpers.classes.mixin.has_two_steps_init import HasTwoStepInit


class AbstractAddonManager(
    AbstractKernelChild,
    HasTwoStepInit,
    HasSnakeShortClassNameClassMixin,
    BaseModel,
    WithWorkdirMixin,
):
    def __init__(self, kernel: "AbstractKernel", **kwargs):
        import inspect
        import os.path

        BaseModel.__init__(self, **kwargs)
        AbstractKernelChild.__init__(self, kernel=kernel)
        
        # Get the path of the actual addon manager class file
        manager_file = inspect.getfile(self.__class__)
        
        WithWorkdirMixin._init_workdir(
            self,
            entrypoint_path=os.path.dirname(manager_file),
            io_manager=self.kernel.io
        )

    @classmethod
    def get_class_name_suffix(cls) -> Optional[str]:
        return "AddonManager"



from wexample_helpers.const.types import UPGRADE_TYPE_MINOR
from wexample_helpers.helpers.version import version_increment
from wexample_wex_core.common.kernel import Kernel


def default__version__increment(
        kernel: "Kernel",
        version: str,
        type: str = UPGRADE_TYPE_MINOR,
        increment: int = 1,
        build: bool = False,
) -> str:
    return version_increment(
        version=version,
        type=type,
        increment=increment,
        build=build,
    )



CORE_COMMAND_NAME = "wex"
WORKDIR_SETUP_DIR = f".{CORE_COMMAND_NAME}"

# Command types
COMMAND_TYPE_SERVICE = "service"
COMMAND_TYPE_ADDON = "addon"

# Command patterns
COMMAND_PATTERN_SERVICE = r"^@([\w_-]+)::([\w_-]+)/([\w_-]+)$"
COMMAND_PATTERN_ADDON = r"^(?:([\w_-]+)::)?([\w_-]+)/([\w_-]+)$"

COMMAND_SEPARATOR_FUNCTION_PARTS = "__"


from typing import Optional, TYPE_CHECKING

from wexample_app.resolver.abstract_command_resolver import AbstractCommandResolver
from wexample_helpers.const.globals import FILE_EXTENSION_PYTHON
from wexample_wex_core.const.globals import COMMAND_TYPE_SERVICE, COMMAND_PATTERN_SERVICE

if TYPE_CHECKING:
    from wexample_app.common.command_request import CommandRequest


class ServiceCommandResolver(AbstractCommandResolver):
    @classmethod
    def get_pattern(cls) -> str:
        return COMMAND_PATTERN_SERVICE

    @classmethod
    def get_type(cls) -> str:
        return COMMAND_TYPE_SERVICE

    def build_command_path(self, request: "CommandRequest") -> Optional[str]:
        return f"{self.kernel.workdir.get_resolved()}cli/{request.name}.{FILE_EXTENSION_PYTHON}"

    def build_command_function_name(self, request: "CommandRequest") -> Optional[str]:
        import re
        return re.sub(r'[^a-zA-Z0-9_]', '', request.name.replace("/", "__"))



from pathlib import Path
from typing import Optional, TYPE_CHECKING, cast

from wexample_app.resolver.abstract_command_resolver import AbstractCommandResolver
from wexample_helpers.const.globals import FILE_EXTENSION_PYTHON
from wexample_helpers.helpers.string import string_to_snake_case
from wexample_wex_core.const.globals import COMMAND_TYPE_ADDON, COMMAND_PATTERN_ADDON, COMMAND_SEPARATOR_FUNCTION_PARTS

if TYPE_CHECKING:
    from wexample_app.common.command_request import CommandRequest


class AddonCommandResolver(AbstractCommandResolver):
    @classmethod
    def get_pattern(cls) -> str:
        return COMMAND_PATTERN_ADDON

    @classmethod
    def get_type(cls) -> str:
        return COMMAND_TYPE_ADDON

    def build_command_path(self, request: "CommandRequest") -> Optional[str]:
        from wexample_wex_core.common.abstract_addon_manager import AbstractAddonManager

        match = request.match

        # Extract addon, group and command from match
        addon = string_to_snake_case(match.group(1))
        group = string_to_snake_case(match.group(2))
        command = string_to_snake_case(match.group(3))

        addon_manager = cast(AbstractAddonManager, self.kernel.get_registry('addon').get(addon))

        return str(
            Path(addon_manager.workdir.get_resolved())
            / "instructions"
            / group
            / f"{command}.{FILE_EXTENSION_PYTHON}"
        )

    def build_command_function_name(self, request: "CommandRequest") -> Optional[str]:
        return string_to_snake_case(
            COMMAND_SEPARATOR_FUNCTION_PARTS.join(self.get_function_name_parts(
                request.match.groups()
            ))
        )



from typing import Type, TYPE_CHECKING, Optional, List, cast

from wexample_app.common.abstract_kernel import AbstractKernel
from wexample_app.common.mixins.command_line_kernel import CommandLineKernel
from wexample_app.common.mixins.command_runner_kernel import CommandRunnerKernel
from wexample_prompt.enums.verbosity_level import VerbosityLevel
from wexample_wex_core.resolver.addon_command_resolver import AddonCommandResolver

if TYPE_CHECKING:
    from wexample_app.resolver.abstract_command_resolver import AbstractCommandResolver
    from wexample_filestate.file_state_manager import FileStateManager
    from wexample_wex_core.common.abstract_addon_manager import AbstractAddonManager


class Kernel(CommandRunnerKernel, CommandLineKernel, AbstractKernel):
    def __init__(self, **kwargs) -> None:
        AbstractKernel.__init__(self, **kwargs)

    def setup(self, addons: Optional[List[Type["AbstractAddonManager"]]] = None) -> "AbstractKernel":
        response = super().setup()
        self._init_addons(addons=addons)
        self._init_command_line_kernel()
        self._init_resolvers()
        self._init_runners()

        return response

    def _init_addons(self, addons: Optional[List[Type["AbstractAddonManager"]]] = None):
        from wexample_app.service.service_registry import ServiceRegistry

        cast(ServiceRegistry, self.set_registry("addon"))
        registry = self.register_items("addon", addons or [])
        registry.instantiate_all(kernel=self)

    def _get_command_resolvers(self) -> list[Type["AbstractCommandResolver"]]:
        from wexample_wex_core.resolver.service_command_resolver import ServiceCommandResolver

        return [
            AddonCommandResolver,
            ServiceCommandResolver,
        ]

    def _get_workdir_state_manager_class(self) -> Type["FileStateManager"]:
        from wexample_wex_core.workdir.kernel_workdir import KernelWorkdir

        return KernelWorkdir

    def _get_core_args(self):
        return super()._get_core_args() + [
            {"arg": "quiet", "attr": "verbosity", "value": VerbosityLevel.QUIET},
            {"arg": "vv", "attr": "verbosity", "value": VerbosityLevel.MEDIUM},
            {"arg": "vvv", "attr": "verbosity", "value": VerbosityLevel.MAXIMUM},
        ]



from wexample_config.const.types import DictConfig
from wexample_filestate.file_state_manager import FileStateManager
from wexample_helpers.const.types import *


class Workdir(FileStateManager):
    should_exist: bool = True

    def prepare_value(self, config: Optional[DictConfig] = None) -> DictConfig:
        from wexample_wex_core.const.globals import WORKDIR_SETUP_DIR
        from wexample_filestate.const.disk import DiskItemType

        config = super().prepare_value(config)

        if "children" not in config:
            config["children"] = []

        config["children"].append({
            "name": WORKDIR_SETUP_DIR,
            'type': DiskItemType.DIRECTORY,
            'should_exist': True,
        })

        return config



from wexample_config.const.types import DictConfig
from wexample_filestate.const.disk import DiskItemType
from wexample_filestate_dev.workdir.mixins.with_readme_workdir_mixin import WithReadmeWorkdirMixin
from wexample_filestate_dev.workdir.mixins.with_version_workdir_mixin import WithVersionWorkdirMixin
from wexample_helpers.const.types import *

from wexample_wex_addon_app.const.globals import (
    APP_FILE_APP_CONFIG,
    APP_FILE_APP_ENV, APP_DIR_APP_DATA_NAME,
)
from wexample_wex_core.workdir.workdir import Workdir


class ProjectWorkdir(WithReadmeWorkdirMixin, WithVersionWorkdirMixin, Workdir):
    def prepare_value(self, config: Optional[DictConfig] = None) -> DictConfig:
        from wexample_config.config_value.filter.trim_config_value_filter import TrimConfigValueFilter

        config = super().prepare_value(config)

        config.update({
            "mode": "777",
            "mode_recursive": True,
        })

        children = config["children"]

        self.append_readme(config=config)
        self.append_version(config=config)

        children.append({
            "name": '.gitignore',
            "type": DiskItemType.FILE,
            "should_exist": True,
            "content_filter": TrimConfigValueFilter
        })

        children.append({
            "name": APP_DIR_APP_DATA_NAME,
            "type": DiskItemType.DIRECTORY,
            "should_exist": True,
            "children": [
                {
                    "name": APP_FILE_APP_ENV,
                    "type": DiskItemType.FILE,
                    "should_exist": True
                },
                {
                    "name": APP_FILE_APP_CONFIG,
                    "type": DiskItemType.FILE,
                    "should_exist": True
                },
                {
                    "name": "tmp",
                    "type": DiskItemType.DIRECTORY,
                    "should_exist": True
                }
            ]
        })

        return config



from wexample_wex_core.common.kernel import Kernel


def default__info__show(
        kernel: "Kernel",
        arguments # TODO TMP
) -> None:
    from wexample_app.const.globals import ENV_VAR_NAME_APP_ENV

    kernel.io.properties(
        title="General",
        properties={
            "Location": kernel.workdir.get_resolved(),
            "Environment": kernel.get_env_parameter(ENV_VAR_NAME_APP_ENV),
            "Arguments": kernel._sys_argv,
        }
    )

    kernel.io.properties(
        title="Resolvers",
        properties=kernel.get_resolvers()
    )

    kernel.io.properties(
        title="Runners",
        properties=kernel.get_runners()
    )
